/*jslint white: false, nomen: false, onevar: false, forin: false, browser: true, devel: true */
/*global jQuery window */

/** @license
 *  ECMWF Container
 *  Copyright ECMWF http://www.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 *  The ECMWF Container widget allows for nested elements whose size is set by
 *  javascript to receive resize type events that propogate down from the root
 *  element.
 *
 *  Window resize
 *     For window resize events to be propagated, body must be a container.
 *
 *  The 'layout' event:
 *     This is to tell child *containers* to carry out layout. Regular child elements should not use this event, but should use 'resize' instead.
 *     Senders must specify a size.
 *
 */

define(['jquery', 'throttle-debounce'], function($) {

    var margins = [
        'marginTop', 'marginBottom', 'marginLeft', 'marginRight',
        'paddingTop', 'paddingBottom', 'paddingLeft', 'paddingRight'
    ];

    function adjust(elem, size) {

        var m = {};
        $.each(margins, function(i, name) {
            m[name] = parseFloat(elem.css(name)) || 0;
        });

        size.width  -= m.marginLeft + m.marginRight + m.paddingLeft + m.paddingRight;
        if(size.height) size.height -= m.marginTop + m.marginBottom + m.paddingTop + m.paddingBottom;

        return size;
    }

    var engines = {
        single: function(elem, options) {
            return {
                layout: function single(size) {

                    var s = {height: size.height, width: size.width};
                    adjust(elem, s);

                    elem
                        .css({height: s.height})
                        .attr('sizor', 'single')
                        .resize();

                    elem.children().content().triggerHandler('layout', s);
                }
            };
        },

        horizontal: function(elem, options) {

            var expandable = options.expandable;

            return {
                layout : function horizontal(size) {
                    size || (size = {width: elem.parent().width()}); //size arg may be missing when layout is triggered externally.
                    var width = size.width;

                    elem.children(':visible').each(function(){
                        if(this != expandable[0] && this.nodeName !== 'BR'){
                            width -= $(this).outerWidth(true);
                        }
                    });

                    var s = {height: size.height, width: width - parseFloat(elem.css('padding-left')) - parseFloat(elem.css('padding-right'))};
                    var expandable_size = {width: adjust(expandable, s).width - 1};
                    expandable
                        .css({width: expandable_size.width})
                        .trigger('resize', expandable_size)       // notify the expandable
                        .content().triggerHandler('layout', s);   // recurse down to child containers.

                    //fire an additional event that explicitly provides the new dimensions to the expandable
                    expandable.triggerHandler('container', expandable_size);
                }
            };
        },

        grid: function(elem, options) {
            return {
                layout : function grid(size) {

                    var w = size.width  / options.cols;
                    var h = size.height / options.rows;
                    var x = 0, y = 0;
                    var n = 0;

                    $.each(options.items,function(i,item) {

                        item = $(item);

                        var size = {width: Math.floor(w), height: Math.floor(h) };
                        var loc  = {top: Math.floor(y), left: Math.floor(x)};

                        adjust(item,size);

                        item
                            .css($.extend({position: 'absolute'}, loc, size))
                            .resize()
                            .content()
                            .triggerHandler('layout', size);

                        x += w;
                        n++;

                        if(n >= options.cols) {
                            n = 0;
                            x = 0;
                            y += h;
                        }
                    });
                }
            };
        },

        // this layout serves as an intermediary - it does not do any sizing itself.
        root: function(elem, options) {
            return {
                layout: function single(size) {
                    size && elem.children().content().triggerHandler('layout', adjust(elem, {height: size.height, width: size.width}));
                }
            };
        }
    };


var size = {};
function doLayout(event, o) {
    o || (o = {});
    var root = (o.root || $('body'));
    var el = (o.root || $(window));
    if(root.hasClass('containr')) {
        var new_size = {width: el.width(), height: el.height()}; //size is sometimes passed as argument. maybe we should use it.
        if(new_size.height !== size.height || new_size.width !== size.width){ //will only work if root is the same as previously
            var _size = $.extend({}, new_size);

            //if(!o.root) console.log("DOLAYOUT WINDOW " + size.width + ' x ' + size.height);

            adjust(root, _size);
            if(!o.root){
                root.css({width: 'auto', height: _size.height});
            }
            root.triggerHandler('layout', _size);
            size = new_size;
        }
    }
}

// WebKit and IE send a continuous series of resize events
$(window).resize($.debounce(250, doLayout));


// Return theclosed container, starting from this
// we want to return all child containers that dont have any other parent containers. This function only aproximates that.
$.fn.content = function() {

    var children;
    if(this.hasClass('containr')){
        return this;
    }else if((children = this.find('> .containr')).length) {
        return children;
    }else {
        return this.find('.containr:first');
    }
};

$.fn.container = function(options) {

    if(options === 'destroy'){
        this.removeData('container').off('layout');
        return;
    }

    this.each(function() {
        var elem = $(this);

        var engine = ($.isFunction(options.layout)
					? options.layout
					: engines[options.layout]
				)(elem, options);

        elem.addClass('containr');
        elem.data('container', options);

        elem.on('layout', function(e, size) {
            // size arg is optional.

            //console.log('LAYOUT ', elem.attr('class').replace(' container', '')/*.replace('container', '------ BODY ------')*/, elem, (size && size.width) + ' x ' + (size && size.height));
            var data = elem.data('container');
            var _size = data.size || {};

            if(elem.is(':visible')){
                if(!size || (size.height !== _size.height) || (size.width !== _size.width)){
                    setTimeout(function(){
                        engine.layout(size);
                    }, 0);

                    if(size){
                        if(size.width) _size.width = size.width;
                        if(size.height) _size.height = size.height;
                        elem.data('container', $.extend(data, {size: _size}));
                    }
                }
            }else{
                _size = {};
            }

            return false; // Prevent event from bubbling
        });
    });

    //re-layout the container tree, but let the caller finish setting up the tree first.
    setTimeout(function(){
        doLayout(null, {root: options.root});
    }, 5);

    return this;
};

return {
	adjust: adjust,
	show_tree: __DEV__ ? function() {
		console.log('container tree:');

		function next(node) {
			return node.find('.containr').not(seen).first();
		}

		var depth = 1;
		var seen = $([]);

		function show_subtree(node) {
			depth++;
			var root = node;
			while((node = next(root)).length){
				console.log(Array(depth).join('____'), node[0])
				seen = seen.add(node);
				show_subtree(node)
			}
			depth--;
		}

		show_subtree($('body'));
	} : $.noop
}

});
